@@ -8,6 +8,7 @@ Perform Machine Learning algorithms on Feature Matrices.
 import math
 import numpy
 import tempfile
+import operator
 import subprocess
 import namedmatrix as nm
 
@@ -21,6 +22,111 @@ WEKA_CLASSPATH = "/Applications/weka-3-6-1.app/Contents/Resources/Java/weka.jar"
 class ImplementationMissing(Exception):
     pass
 
+
+class FeatureSelection(object):
+    """
+    A general class to perform feature selection of the 
+    example data.
+    """
+    
+    def __init__(self, examples, labels, mlclass, max_features=None):
+        """
+        @param: examples    NamedMatrix of feature vectors, each row is a training example.
+        @param: labels      Classification labels, a list of integer or boolean values.
+        @param: mlclass     A MachineLearning subclass.
+        """
+        
+        self.examples = examples
+        self.labels = labels
+        self.mlclass = mlclass
+        if type(max_features) == int:
+            self.max_features = max_features
+        elif type(max_features) == float:
+            self.max_features = int(max_features * self.examples.size(0))
+        elif max_features is None:
+            self.max_features = self.examples.size(1)
+        
+        self.folds = None
+        
+    
+    def divide_examples(self, num_folds):
+        """
+        Divide the training examples into num_folds.
+        """
+        rows = self.examples.rownames
+        folds_names = dict()
+        folds_index = dict()
+        
+        for i,row in enumerate(rows):
+            rand_fold = random.randint(0,num_folds-1)
+            
+            if not rand_fold in folds_names:
+                folds_names[rand_fold] = list()
+            if not rand_fold in folds_index:
+                folds_index[rand_fold] = list()
+            
+            folds_names[rand_fold].append(row)
+            folds_index[rand_fold].append(i)
+        
+        self.folds = dict()
+        self.folds['names'] = folds_names
+        self.folds['index'] = folds_index
+    
+    def forward_selection(self, num_folds=5, statistical_test="wilcox.test", args = None):
+        """
+        Perform forward selection and return a mlclass classifier trained
+        with those features.
+        @param: folds               The number-fold cross-validation to perform.
+        @param: statistica_test     The type of statistical test to perform. Requires rpy2 and R to be installed.
+        """
+        import rpy2.robjects as robjects
+        
+        self.divide_examples(num_folds)
+        feature_sets = list()
+        for i in range(num_folds):
+            
+            train_rows = reduce(operator.add, [fold for k,fold in self.folds['names'].items() if not k == i])
+            train_data = self.examples[train_rows,:]
+            train_index = reduce(operator.add, [fold for k,fold in self.folds['index'].items() if not k == i])
+            train_labels = [self.labels[k] for k in train_index]
+            
+            test_rows = reduce(operator.add, [fold for k,fold in self.folds['names'].items() if k == i])
+            test_index = reduce(operator.add, [fold for k,fold in self.folds['index'].items() if k == i])
+            test_data = self.examples[test_rows,:]
+            test_labels = [self.labels[k] for k in test_index]
+            
+            pvalues = list()
+            for j,feat in enumerate(self.examples.colnames):
+                
+                if statistical_test == "wilcox.test":
+                    positive_values = ",".join(map(str,[train_data[k,j] for k in range(train_data.size(0)) if train_labels[k] == 1]))
+                    negative_values = ",".join(map(str,[train_data[k,j] for k in range(train_data.size(0)) if train_labels[k] == 0]))
+                    pvalue = robjects.r("wilcox.test(c(%s), c(%s))$p.value" % (positive_values, negative_values))[0]
+                
+                pvalues.append( (pvalue,feat,j) )
+            
+            selected = [feat for p,feat,j in sorted(pvalues) if p < 0.05][:self.max_features]
+            
+            best_features = None
+            print >> sys.stderr, "Fold %d: Found %d enriched features." % (i, len(selected))
+            for k in range(len(selected)):
+                
+                train = train_data[:,selected[:k]]
+                test = test_data[:,selected[:k]]
+                
+                mlobj = self.mlclass(train, train_labels)
+                train_results = mlobj.cross_validate()
+                test_results = mlobj.test(test, test_labels)
+                
+                print >> sys.stderr, "  %d    %f  %f  %f" % (k, train_results['AUROC'], train_results['TrainingError'], test_results['TestError'])
+                
+                if best_features is None or test_results['TestError'] < best_features[0]:
+                    best_features = (test_results['TestError'], selected[:k])
+            
+            feature_sets.append( (best_features[0], best_features[1]) )
+        
+        return sorted(feature_sets)[0][1]
+
 class MachineLearning(object):
     """
     A abstract Machine Learning object. A ML object is not instantiated
